#include "envy_engine.h"
#include <stdlib.h>
#include <string.h>

/* Private */
static Ecore_Evas *
_ecore_evas_buffer_new_warp(const char * disp_name,
                            Ecore_X_Window parent,
                            int x, int y,
                            int w, int h)
{
  return ecore_evas_buffer_new(w, h);
}

#if 0
static Ecore_Evas *
_ecore_evas_fb_new_warp(const char *disp_name,
                        Ecore_X_Window parent,
                        int x, int y,
                        int w, int h)
{
  return ecore_evas_fb_new((char*)disp_name, 0, w, h);
}

static Ecore_Evas *
_ecore_evas_directfb_new_warp(const char *disp_name,
                             Ecore_X_Window parent,
                             int x, int y,
                             int w, int h)
{
  return ecore_evas_directfb_new(disp_name, 0, x, y, w, h);
}

static Ecore_Evas *
_ecore_evas_sdl_new_warp(const char *dispname,
                         Ecore_X_Window parent,
                         int x, int y,
                         int w, int h)
{
  return ecore_evas_sdl_new(dispname, w, h, 0, 0, 0, 0);
}
#endif

struct _Envy_Engine_Desc
{
  Ecore_Evas_Engine_Type  type;
  const char             *name;
  const char             *desc;
  Ecore_Evas     * (*new)        (const char*, Ecore_X_Window, int, int, int, int);
  Ecore_X_Window   (*window_get) (Ecore_Evas*);
};
typedef struct _Envy_Engine_Desc Envy_Engine_Desc;


static const Envy_Engine_Desc _envy_engines[] =
{
    { ECORE_EVAS_ENGINE_SOFTWARE_BUFFER, 
      "software_buffer", 
      "SOFTWARE BUFFER",
      _ecore_evas_buffer_new_warp, 
      NULL
    },
    { ECORE_EVAS_ENGINE_SOFTWARE_X11, 
      "software_x11",
      "SOFTWARE X11",
      ecore_evas_software_x11_new,
      ecore_evas_software_x11_window_get
    },
    { ECORE_EVAS_ENGINE_XRENDER_X11, 
      "xrender_x11", 
      "XRENDER X11",
      ecore_evas_xrender_x11_new, 
      ecore_evas_xrender_x11_window_get
    },
    { ECORE_EVAS_ENGINE_OPENGL_X11, 
      "opengl_x11", 
      "OPENGL X11",
      ecore_evas_gl_x11_new, 
      ecore_evas_gl_x11_window_get
    },
#if 0
    { ECORE_EVAS_ENGINE_SOFTWARE_FB, 
      "software_fb", 
      "SOFTWARE FB",
      _ecore_evas_fb_new_warp, 
      NULL
    },
    { ECORE_EVAS_ENGINE_DIRECTFB, 
      "directfb", 
      "DIRECTFB",
      _ecore_evas_directfb_new_warp, 
      NULL
    },
    { ECORE_EVAS_ENGINE_SOFTWARE_16_X11, 
      "software_16_x11", 
      "SOFTWARE 16 X11",
      ecore_evas_software_x11_16_new, 
      ecore_evas_software_x11_16_window_get
    }
    { ECORE_EVAS_ENGINE_SDL, 
      "sdl", 
      "SDL",
      _ecore_evas_sdl_new_warp, 
      NULL
    }
#endif
};
static const size_t _envy_engines_nb = sizeof(_envy_engines) / sizeof(Envy_Engine_Desc);

/* Public */
const char *
envy_engine_name(Ecore_Evas_Engine_Type engine)
{
  if (engine >= _envy_engines_nb)
    return NULL;
  else
    return _envy_engines[engine].name;
}

const char **
envy_engine_names(void)
{
  size_t i, j;
  const char **names = NULL;

  names = calloc(_envy_engines_nb + 1, sizeof(char*));
  if ( names == NULL ) return NULL;

  for (i = 0, j = 0; i < _envy_engines_nb ; i++)
    {
      if (ecore_evas_engine_type_supported_get(_envy_engines[i].type))
        {
          names[j] = _envy_engines[i].name;
          j++;
        }
    }
  return names;
}

const char *
envy_engine_desc(Ecore_Evas_Engine_Type engine)
{
  if (engine >= _envy_engines_nb)
    return NULL;
  else
    return _envy_engines[engine].desc;
}

Ecore_Evas_Engine_Type
envy_engine_type(const char *name)
{
  size_t i;

  for (i = 0; i < _envy_engines_nb; i++)
    {
      if (!strcmp(_envy_engines[i].name, name))
        return _envy_engines[i].type;
    }

  return -1;
}

Ecore_Evas *
envy_engine_new(Ecore_Evas_Engine_Type engine,
                const char *disp_name, Ecore_X_Window parent,
                int x, int y,
                int w, int h)
{
  if ( engine >= _envy_engines_nb )
    return NULL;

  return _envy_engines[engine].new(disp_name, parent, x, y, w, h);
}

Ecore_X_Window
envy_engine_window_get(Ecore_Evas_Engine_Type engine,
                       Ecore_Evas *ee)
{
  if ( engine >= _envy_engines_nb || _envy_engines[engine].window_get == NULL )
    return 0;

  return _envy_engines[engine].window_get(ee);
}
